# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from test_uai_attributes import attributes_raillabel_dict, attributes_uai, attributes_uai_dict
from test_uai_bounding_box_2d import (
    bounding_box_2d_raillabel_dict,
    bounding_box_2d_uai,
    bounding_box_2d_uai_dict,
)
from test_uai_bounding_box_3d import (
    bounding_box_3d_raillabel_dict,
    bounding_box_3d_uai,
    bounding_box_3d_uai_dict,
)
from test_uai_coordinate_system import (
    coordinate_system_camera_raillabel_dict,
    coordinate_system_camera_translated_uid,
    coordinate_system_camera_uai,
    coordinate_system_camera_uai_dict,
    coordinate_system_lidar_raillabel_dict,
    coordinate_system_lidar_translated_uid,
    coordinate_system_lidar_uai,
    coordinate_system_lidar_uai_dict,
    coordinate_system_radar_raillabel_dict,
    coordinate_system_radar_translated_uid,
    coordinate_system_radar_uai,
    coordinate_system_radar_uai_dict,
)
from test_uai_frame import frame_raillabel_dict, frame_uai, frame_uai_dict
from test_uai_metadata import metadata_raillabel_dict, metadata_uai, metadata_uai_dict
from test_uai_point_3d import point_3d_uai, point_3d_uai_dict, point_3d_vec
from test_uai_polygon_2d import polygon_2d_raillabel_dict, polygon_2d_uai, polygon_2d_uai_dict
from test_uai_polyline_2d import polyline_2d_raillabel_dict, polyline_2d_uai, polyline_2d_uai_dict
from test_uai_quaternion import quaternion_uai, quaternion_uai_dict, quaternion_vec
from test_uai_scene import scene_raillabel_dict, scene_uai, scene_uai_dict
from test_uai_segmentation_3d import (
    segmentation_3d_raillabel_dict,
    segmentation_3d_uai,
    segmentation_3d_uai_dict,
)
from test_uai_sensor_reference import (
    sensor_camera_raillabel_dict,
    sensor_camera_uai,
    sensor_camera_uai_dict,
    sensor_lidar_raillabel_dict,
    sensor_lidar_uai,
    sensor_lidar_uai_dict,
)
from test_uai_size_3d import size_3d_uai, size_3d_uai_dict, size_3d_vec
